package org.apache.bcel.generic;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import org.apache.bcel.classfile.AnnotationEntry;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.ElementValuePair;
import org.apache.bcel.classfile.RuntimeInvisibleAnnotations;
import org.apache.bcel.classfile.RuntimeInvisibleParameterAnnotations;
import org.apache.bcel.classfile.RuntimeVisibleAnnotations;
import org.apache.bcel.classfile.RuntimeVisibleParameterAnnotations;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.ElementValueGen;
import org.apache.bcel.generic.ElementValuePairGen;
import org.apache.bcel.generic.ObjectType;

public class AnnotationEntryGen {
  static final org.apache.bcel.generic.AnnotationEntryGen[] EMPTY_ARRAY = new org.apache.bcel.generic.AnnotationEntryGen[0];
  
  private int typeIndex;
  
  private List<ElementValuePairGen> evs;
  
  private final ConstantPoolGen cpool;
  
  private boolean isRuntimeVisible;
  
  static Attribute[] getAnnotationAttributes(ConstantPoolGen cp, org.apache.bcel.generic.AnnotationEntryGen[] annotationEntryGens) {
    if (annotationEntryGens.length == 0)
      return Attribute.EMPTY_ARRAY; 
    try {
      int countVisible = 0;
      int countInvisible = 0;
      for (org.apache.bcel.generic.AnnotationEntryGen a : annotationEntryGens) {
        if (a.isRuntimeVisible()) {
          countVisible++;
        } else {
          countInvisible++;
        } 
      } 
      ByteArrayOutputStream rvaBytes = new ByteArrayOutputStream();
      ByteArrayOutputStream riaBytes = new ByteArrayOutputStream();
      try(DataOutputStream rvaDos = new DataOutputStream(rvaBytes); DataOutputStream riaDos = new DataOutputStream(riaBytes)) {
        rvaDos.writeShort(countVisible);
        riaDos.writeShort(countInvisible);
        for (org.apache.bcel.generic.AnnotationEntryGen a : annotationEntryGens) {
          if (a.isRuntimeVisible()) {
            a.dump(rvaDos);
          } else {
            a.dump(riaDos);
          } 
        } 
      } 
      byte[] rvaData = rvaBytes.toByteArray();
      byte[] riaData = riaBytes.toByteArray();
      int rvaIndex = -1;
      int riaIndex = -1;
      if (rvaData.length > 2)
        rvaIndex = cp.addUtf8("RuntimeVisibleAnnotations"); 
      if (riaData.length > 2)
        riaIndex = cp.addUtf8("RuntimeInvisibleAnnotations"); 
      List<Attribute> newAttributes = new ArrayList<>();
      if (rvaData.length > 2)
        newAttributes
          .add(new RuntimeVisibleAnnotations(rvaIndex, rvaData.length, new DataInputStream(new ByteArrayInputStream(rvaData)), cp.getConstantPool())); 
      if (riaData.length > 2)
        newAttributes.add(new RuntimeInvisibleAnnotations(riaIndex, riaData.length, new DataInputStream(new ByteArrayInputStream(riaData)), cp
              .getConstantPool())); 
      return newAttributes.<Attribute>toArray(Attribute.EMPTY_ARRAY);
    } catch (IOException e) {
      System.err.println("IOException whilst processing annotations");
      e.printStackTrace();
      return null;
    } 
  }
  
  static Attribute[] getParameterAnnotationAttributes(ConstantPoolGen cp, List<org.apache.bcel.generic.AnnotationEntryGen>[] vec) {
    int[] visCount = new int[vec.length];
    int totalVisCount = 0;
    int[] invisCount = new int[vec.length];
    int totalInvisCount = 0;
    try {
      for (int i = 0; i < vec.length; i++) {
        if (vec[i] != null)
          for (org.apache.bcel.generic.AnnotationEntryGen element : vec[i]) {
            if (element.isRuntimeVisible()) {
              visCount[i] = visCount[i] + 1;
              totalVisCount++;
              continue;
            } 
            invisCount[i] = invisCount[i] + 1;
            totalInvisCount++;
          }  
      } 
      ByteArrayOutputStream rvaBytes = new ByteArrayOutputStream();
      try (DataOutputStream rvaDos = new DataOutputStream(rvaBytes)) {
        rvaDos.writeByte(vec.length);
        for (int j = 0; j < vec.length; j++) {
          rvaDos.writeShort(visCount[j]);
          if (visCount[j] > 0)
            for (org.apache.bcel.generic.AnnotationEntryGen element : vec[j]) {
              if (element.isRuntimeVisible())
                element.dump(rvaDos); 
            }  
        } 
      } 
      ByteArrayOutputStream riaBytes = new ByteArrayOutputStream();
      try (DataOutputStream riaDos = new DataOutputStream(riaBytes)) {
        riaDos.writeByte(vec.length);
        for (int j = 0; j < vec.length; j++) {
          riaDos.writeShort(invisCount[j]);
          if (invisCount[j] > 0)
            for (org.apache.bcel.generic.AnnotationEntryGen element : vec[j]) {
              if (!element.isRuntimeVisible())
                element.dump(riaDos); 
            }  
        } 
      } 
      byte[] rvaData = rvaBytes.toByteArray();
      byte[] riaData = riaBytes.toByteArray();
      int rvaIndex = -1;
      int riaIndex = -1;
      if (totalVisCount > 0)
        rvaIndex = cp.addUtf8("RuntimeVisibleParameterAnnotations"); 
      if (totalInvisCount > 0)
        riaIndex = cp.addUtf8("RuntimeInvisibleParameterAnnotations"); 
      List<Attribute> newAttributes = new ArrayList<>();
      if (totalVisCount > 0)
        newAttributes.add(new RuntimeVisibleParameterAnnotations(rvaIndex, rvaData.length, new DataInputStream(new ByteArrayInputStream(rvaData)), cp
              .getConstantPool())); 
      if (totalInvisCount > 0)
        newAttributes.add(new RuntimeInvisibleParameterAnnotations(riaIndex, riaData.length, new DataInputStream(new ByteArrayInputStream(riaData)), cp
              .getConstantPool())); 
      return newAttributes.<Attribute>toArray(Attribute.EMPTY_ARRAY);
    } catch (IOException e) {
      System.err.println("IOException whilst processing parameter annotations");
      e.printStackTrace();
      return null;
    } 
  }
  
  public static org.apache.bcel.generic.AnnotationEntryGen read(DataInput dis, ConstantPoolGen cpool, boolean b) throws IOException {
    org.apache.bcel.generic.AnnotationEntryGen a = new org.apache.bcel.generic.AnnotationEntryGen(cpool);
    a.typeIndex = dis.readUnsignedShort();
    int elemValuePairCount = dis.readUnsignedShort();
    for (int i = 0; i < elemValuePairCount; i++) {
      int nidx = dis.readUnsignedShort();
      a.addElementNameValuePair(new ElementValuePairGen(nidx, ElementValueGen.readElementValue(dis, cpool), cpool));
    } 
    a.isRuntimeVisible(b);
    return a;
  }
  
  public AnnotationEntryGen(AnnotationEntry a, ConstantPoolGen cpool, boolean copyPoolEntries) {
    this.cpool = cpool;
    if (copyPoolEntries) {
      this.typeIndex = cpool.addUtf8(a.getAnnotationType());
    } else {
      this.typeIndex = a.getAnnotationTypeIndex();
    } 
    this.isRuntimeVisible = a.isRuntimeVisible();
    this.evs = copyValues(a.getElementValuePairs(), cpool, copyPoolEntries);
  }
  
  private AnnotationEntryGen(ConstantPoolGen cpool) {
    this.cpool = cpool;
  }
  
  public AnnotationEntryGen(ObjectType type, List<ElementValuePairGen> elements, boolean vis, ConstantPoolGen cpool) {
    this.cpool = cpool;
    this.typeIndex = cpool.addUtf8(type.getSignature());
    this.evs = elements;
    this.isRuntimeVisible = vis;
  }
  
  public void addElementNameValuePair(ElementValuePairGen evp) {
    if (this.evs == null)
      this.evs = new ArrayList<>(); 
    this.evs.add(evp);
  }
  
  private List<ElementValuePairGen> copyValues(ElementValuePair[] in, ConstantPoolGen cpool, boolean copyPoolEntries) {
    List<ElementValuePairGen> out = new ArrayList<>();
    for (ElementValuePair nvp : in)
      out.add(new ElementValuePairGen(nvp, cpool, copyPoolEntries)); 
    return out;
  }
  
  public void dump(DataOutputStream dos) throws IOException {
    dos.writeShort(this.typeIndex);
    dos.writeShort(this.evs.size());
    for (ElementValuePairGen envp : this.evs)
      envp.dump(dos); 
  }
  
  public AnnotationEntry getAnnotation() {
    AnnotationEntry a = new AnnotationEntry(this.typeIndex, this.cpool.getConstantPool(), this.isRuntimeVisible);
    for (ElementValuePairGen element : this.evs)
      a.addElementNameValuePair(element.getElementNameValuePair()); 
    return a;
  }
  
  public int getTypeIndex() {
    return this.typeIndex;
  }
  
  public final String getTypeName() {
    return getTypeSignature();
  }
  
  public final String getTypeSignature() {
    ConstantUtf8 utf8 = (ConstantUtf8)this.cpool.getConstant(this.typeIndex);
    return utf8.getBytes();
  }
  
  public List<ElementValuePairGen> getValues() {
    return this.evs;
  }
  
  public boolean isRuntimeVisible() {
    return this.isRuntimeVisible;
  }
  
  private void isRuntimeVisible(boolean b) {
    this.isRuntimeVisible = b;
  }
  
  public String toShortString() {
    StringBuilder s = new StringBuilder();
    s.append("@").append(getTypeName()).append("(");
    for (int i = 0; i < this.evs.size(); i++) {
      s.append(this.evs.get(i));
      if (i + 1 < this.evs.size())
        s.append(","); 
    } 
    s.append(")");
    return s.toString();
  }
  
  public String toString() {
    StringBuilder s = new StringBuilder(32);
    s.append("AnnotationGen:[").append(getTypeName()).append(" #").append(this.evs.size()).append(" {");
    for (int i = 0; i < this.evs.size(); i++) {
      s.append(this.evs.get(i));
      if (i + 1 < this.evs.size())
        s.append(","); 
    } 
    s.append("}]");
    return s.toString();
  }
}
